#include <stdio.h>
#include <unistd.h>

#include <iostream>
#include <string>

#include <cmvision/Blobs.h>

#include <geometry_msgs/Twist.h>
#include <nao_msgs/JointAnglesWithSpeed.h>
#include <nao_msgs/SonarOdometry.h>
#include <body_msgs/Hand.h>

#include <ros/topic.h>
#include "ros/ros.h"
#include "std_msgs/String.h"

using namespace std;

string server_ip;
string server_port;


/**
 * WALKFORWARD
 */
void walkForward(ros::Publisher pub)
{

	geometry_msgs::Twist msg_walk;

	msg_walk.linear.x 	= 1;
	msg_walk.linear.y 	= 0;
	msg_walk.linear.z 	= 0;
	msg_walk.angular.x 	= 0;	
	msg_walk.angular.y 	= 0;
	msg_walk.angular.z 	= 0;

	pub.publish(msg_walk);
}

/**
 * WALKBACKWARDS
 */
void walkBackwards(ros::Publisher pub)
{

	geometry_msgs::Twist msg_walk;
  	
	msg_walk.linear.x 	= -1;
	msg_walk.linear.y 	= 0;
	msg_walk.linear.z 	= 0;
	msg_walk.angular.x 	= 0;	
	msg_walk.angular.y 	= 0;
	msg_walk.angular.z 	= 0;

	pub.publish(msg_walk);
}

/**
 * WALKSIDEWAYSRIGHT
 */
void walkSidewaysRight(ros::Publisher pub)
{

	geometry_msgs::Twist msg_walk;
  	
	msg_walk.linear.x 	= 0;
	msg_walk.linear.y 	= -1;
	msg_walk.linear.z 	= 0;
	msg_walk.angular.x 	= 0;	
	msg_walk.angular.y 	= 0;
	msg_walk.angular.z 	= 0;

	pub.publish(msg_walk);
}

/**
 * WALKSIDEWAYSLEFT
 */
void walkSidewaysLeft(ros::Publisher pub)
{

	geometry_msgs::Twist msg_walk;
  	
	msg_walk.linear.x 	= 0;
	msg_walk.linear.y 	= 1;
	msg_walk.linear.z 	= 0;
	msg_walk.angular.x 	= 0;	
	msg_walk.angular.y 	= 0;
	msg_walk.angular.z 	= 0;

	pub.publish(msg_walk);
}

/**
 * STOPWALKING
 */
void stopWalking(ros::Publisher pub)
{

	geometry_msgs::Twist msg_walk;
  	
	msg_walk.linear.x 	= 0;
	msg_walk.linear.y 	= 0;
	msg_walk.linear.z 	= 0;
	msg_walk.angular.x 	= 0;	
	msg_walk.angular.y 	= 0;
	msg_walk.angular.z 	= 0;

	pub.publish(msg_walk);
}

/**
 * TURNRIGHT
 */
void turnRight(ros::Publisher pub)
{

	geometry_msgs::Twist msg_walk;
  	
	msg_walk.linear.x 	= 0;
	msg_walk.linear.y 	= 0;
	msg_walk.linear.z 	= 0;
	msg_walk.angular.x 	= 0;	
	msg_walk.angular.y 	= 0;
	msg_walk.angular.z 	= -1;

	pub.publish(msg_walk);
}

/**
 * TURNLEFT
 */
void turnLeft(ros::Publisher pub)
{

	geometry_msgs::Twist msg_walk;

	msg_walk.linear.x 	= 0;
	msg_walk.linear.y 	= 0;
	msg_walk.linear.z 	= 0;
	msg_walk.angular.x 	= 0;	
	msg_walk.angular.y 	= 0;
	msg_walk.angular.z 	= 1;

	pub.publish(msg_walk);
}

/**
 * TURNHEAD
 */
void turnHead(ros::Publisher pub, int angle)
{

	int max_angle= 119.5;
	int min_angle= -119.5;
	float angle_rad= angle*(0.0174532925);

	nao_msgs::JointAnglesWithSpeed msg_turnHead;

	std::vector<std::string> jnames(1,"HeadYaw");
	std::vector<float> jangles(1,angle_rad);

	if(angle>max_angle||angle<min_angle)
	{
		ROS_INFO("Allowed values for angle = [-119,0)U[0,119.5].");
		return;
	}

	msg_turnHead.joint_names= jnames;
	msg_turnHead.joint_angles=jangles;
	msg_turnHead.speed= 0.25;

	pub.publish(msg_turnHead);
}

/**
 * OPENHAND
 */
void openHand(ros::Publisher pub, int hand)
{

	size_t size= 1;

	nao_msgs::JointAnglesWithSpeed msg_openHand;

	std::vector<std::string> jnames(size);
	std::vector<float> jangles(1,1);

	if(hand==1)
	{
		jnames[0]="LHand";
	}
	else if(hand==0)
	{
		jnames[0]="RHand";
	}
	else
	{
		ROS_INFO("Allowed values for hand are 1 (left hand) and 0 (right hand).");
		return;
	}

	msg_openHand.joint_names= jnames;
	msg_openHand.joint_angles=jangles;
	msg_openHand.speed= 1;

	pub.publish(msg_openHand);
}

/**
 * CLOSEHAND
 */
void closeHand(ros::Publisher pub, int hand)
{

	size_t size= 1;

	nao_msgs::JointAnglesWithSpeed msg_closeHand;

	std::vector<std::string> jnames(size);
	std::vector<float> jangles(1,0.4);

	if(hand==1)
	{
		jnames[0]="LHand";
	}
	else if(hand==0)
	{
		jnames[0]="RHand";
	}
	else
	{
		ROS_INFO("Allowed values for hand are 1 (left hand) and 0 (right hand).");
		return;
	}

	msg_closeHand.joint_names= jnames;
	msg_closeHand.joint_angles=jangles;
	msg_closeHand.speed= 1;

	pub.publish(msg_closeHand);
}

/**
 * SAY
 */
void say(ros::Publisher pub, std::string speech)
{
	std_msgs::String msg_speech;

	msg_speech.data=speech;

	pub.publish(msg_speech);
}

/**
 * STRETCHARM
 */
void stretchArm(ros::Publisher pub, int arm, double angle)
{
	//Angle range: [0.5, 89.5]
	size_t size= 1;

	nao_msgs::JointAnglesWithSpeed msg_arm;

	std::vector<std::string> jnames(size);
	std::vector<float> jangles(size);

	if(arm==1)
	{
		jnames[0]="LElbowRoll";
		
		jangles[0]=angle*(0.0174532925);
	}
	else if(arm==0)
	{
		jnames[0]="RElbowRoll";
		
		jangles[0]=angle*(0.0174532925);
	}
	else
	{
		ROS_INFO("Allowed values for hand are 1 (left arm) and 0 (right arm).");
		return;
	}

	msg_arm.joint_names= jnames;
	msg_arm.joint_angles=jangles;
	msg_arm.speed= 0.25;

	pub.publish(msg_arm);
}


/**
 * LIFTARM
 */
void liftArm(ros::Publisher pub, int arm, double angle)
{
	//Angle range: [-119, 119]
	size_t size= 1;

	nao_msgs::JointAnglesWithSpeed msg_arm;

	std::vector<std::string> jnames(size);
	std::vector<float> jangles(size);

	if(arm==1)
	{
		jnames[0]="LShoulderPitch";
		
		jangles[0]=angle*(0.0174532925);
	}
	else if(arm==0)
	{
		jnames[0]="RShoulderPitch";
		
		jangles[0]=angle*(0.0174532925);
	}
	else
	{
		ROS_INFO("Allowed values for hand are 1 (left arm) and 0 (right arm).");
		return;
	}

	msg_arm.joint_names= jnames;
	msg_arm.joint_angles=jangles;
	msg_arm.speed= 0.25;

	pub.publish(msg_arm);
}

/**
 * ROLLARM
 */
void rollArm(ros::Publisher pub, int arm, double angle)
{
	//Angle range: [0.5, 94.5]
	size_t size= 1;

	nao_msgs::JointAnglesWithSpeed msg_arm;

	std::vector<std::string> jnames(size);
	std::vector<float> jangles(size);

	if(arm==1)
	{
		jnames[0]="LShoulderRoll";
		
		jangles[0]=angle*(0.0174532925);
	}
	else if(arm==0)
	{
		jnames[0]="RShoulderRoll";
		
		jangles[0]=angle*(0.0174532925);
	}
	else
	{
		ROS_INFO("Allowed values for hand are 1 (left arm) and 0 (right arm).");
		return;
	}

	msg_arm.joint_names= jnames;
	msg_arm.joint_angles=jangles;
	msg_arm.speed= 0.25;

	pub.publish(msg_arm);
}


/**
 * ROLLWRIST
 */
void rollWrist(ros::Publisher pub, int arm, double angle)
{
	//Angle range: [-104, 104]
	size_t size= 1;

	nao_msgs::JointAnglesWithSpeed msg_arm;

	std::vector<std::string> jnames(size);
	std::vector<float> jangles(size);

	if(arm==1)
	{
		jnames[0]="LWristYaw";
		
		jangles[0]=angle*(0.0174532925);
	}
	else if(arm==0)
	{
		jnames[0]="RWristYaw";
		
		jangles[0]=angle*(0.0174532925);
	}
	else
	{
		ROS_INFO("Allowed values for hand are 1 (left arm) and 0 (right arm).");
		return;
	}

	msg_arm.joint_names= jnames;
	msg_arm.joint_angles=jangles;
	msg_arm.speed= 0.25;

	pub.publish(msg_arm);
}

/**
 * ROLLELBOW
 */
void rollElbow(ros::Publisher pub, int arm, double angle)
{
	//Angle range: [-104, 104]
	size_t size= 1;

	nao_msgs::JointAnglesWithSpeed msg_arm;

	std::vector<std::string> jnames(size);
	std::vector<float> jangles(size);

	if(arm==1)
	{
		jnames[0]="LElbowYaw";
		
		jangles[0]=angle*(0.0174532925);
	}
	else if(arm==0)
	{
		jnames[0]="RElbowYaw";
		
		jangles[0]=angle*(0.0174532925);
	}
	else
	{
		ROS_INFO("Allowed values for hand are 1 (left arm) and 0 (right arm).");
		return;
	}

	msg_arm.joint_names= jnames;
	msg_arm.joint_angles=jangles;
	msg_arm.speed= 0.25;

	pub.publish(msg_arm);
}

/**
 * RECIEVEMESSAGE
 */
void recieveMessage(ros::Publisher pub)
{	
	
	walkForward(pub);

	usleep(1000000);

	stopWalking(pub);

}


void moveFullArm(ros::Publisher pub, int arm, double shoulderRollAngle, double shoulderPitchAngle, double elbowRollAngle)
{
	//Angle range: [0.5, 94.5]
	size_t size= 3;

	nao_msgs::JointAnglesWithSpeed msg_arm;

	std::vector<std::string> jnames(size);
	std::vector<float> jangles(size);

	if(arm==1)
	{
		jnames[0]="LShoulderRoll";
		jnames[1]="LShoulderPitch";
		jnames[2]="LElbowRoll";
		jangles[0]=shoulderRollAngle*(0.0174532925);
		jangles[1]=shoulderPitchAngle*(0.0174532925);
		jangles[2]=elbowRollAngle*(0.0174532925);
	}
	else if(arm==0)
	{
		jnames[0]="RShoulderRoll";
		jnames[1]="RShoulderPitch";
		jnames[2]="RElbowRoll";
		jangles[0]=shoulderRollAngle*(0.0174532925);
		jangles[1]=shoulderPitchAngle*(0.0174532925);
		jangles[2]=elbowRollAngle*(0.0174532925);
	}
	else
	{
		ROS_INFO("Allowed values for hand are 1 (left arm) and 0 (right arm).");
		return;
	}

	msg_arm.joint_names= jnames;
	msg_arm.joint_angles=jangles;
	msg_arm.speed= 0.25;

	pub.publish(msg_arm);
}

/**
 * WALKDISTANCE
 */
void walkDistance(ros::Publisher pub, double distance,int direction)
{	
	
	if(direction==0)
	{
		walkForward(pub);

		usleep(10*distance*1000000);

		stopWalking(pub);

		usleep(1000000);
	}
	else if(direction==1)
	{
		walkBackwards(pub);

		usleep(10*distance*1000000);

		stopWalking(pub);

		usleep(1000000);
	}
}

/**
 * WALKSIDEWAYSDISTANCE
 */
void walkSidewaysDistance(ros::Publisher pub, double distance, int direction)
{
	if(direction == 0)
	{
		walkSidewaysRight(pub);

		usleep(10*distance*1000000);

		stopWalking(pub);

		usleep(1000000);
	}
	else if(direction == 1)
	{
		walkSidewaysLeft(pub);

		usleep(10*distance*1000000);

		stopWalking(pub);

		usleep(1000000);
	}
}	

/**
 * TURNRIGHTANGLE
 */
void turnAngle(ros::Publisher pub, double angle)
{	
	if(angle>0)
	{
		turnRight(pub); 

		usleep(2*angle*0.0305*1000000);

		stopWalking(pub);

		usleep(1500000);
	}
	else
	{
		turnLeft(pub);

		usleep(2*angle*-0.0305*1000000);

		stopWalking(pub);

		usleep(1500000);
	}
}

/**
 * GETSONARINFO
 */
int getSonarInfo(ros::NodeHandle nh)
{

	std::string topic;
	topic= "sonar_odometry";
	bool left=false;
	bool right=false;

	float valL1=ros::topic::waitForMessage<nao_msgs::SonarOdometry>(topic,nh)->left1;
float valL2=ros::topic::waitForMessage<nao_msgs::SonarOdometry>(topic,nh)->left1;
float valL3=ros::topic::waitForMessage<nao_msgs::SonarOdometry>(topic,nh)->left1;
float valL4=ros::topic::waitForMessage<nao_msgs::SonarOdometry>(topic,nh)->left1;
float valL5=ros::topic::waitForMessage<nao_msgs::SonarOdometry>(topic,nh)->left1;
	
	float valL= (valL1+valL2+valL3+valL4+valL5)/5;

float valR1=ros::topic::waitForMessage<nao_msgs::SonarOdometry>(topic,nh)->right1;
float valR2=ros::topic::waitForMessage<nao_msgs::SonarOdometry>(topic,nh)->right1;
float valR3=ros::topic::waitForMessage<nao_msgs::SonarOdometry>(topic,nh)->right1;
float valR4=ros::topic::waitForMessage<nao_msgs::SonarOdometry>(topic,nh)->right1;
float valR5=ros::topic::waitForMessage<nao_msgs::SonarOdometry>(topic,nh)->right1;

	float valR= (valR1+valR2+valR3+valR4+valR5)/5;

	if(valL <1)
	{
		left= true;
	}
	if(valR < 1)
	{
		right= true;
	}

	if(!left && !right)
	{
		return 0;
	}
	else if(left && !right)
	{
		return 1;
	}
	else if(!left && right)
	{
		return 2;
	}
	else if(left && right)
	{
		return 3;
	}
	
	usleep(1500000);
	
	return -1;
}

/**
 * DETECTOBJECT
 */
int detectObject(ros::NodeHandle nh)
{

	std::string topic;
	topic= "blobs";

	if(ros::topic::waitForMessage<cmvision::Blobs>(topic,nh)->blob_count > 0)
	{
		int image_width = ros::topic::waitForMessage<cmvision::Blobs>(topic,nh)->image_width;
		int x= ros::topic::waitForMessage<cmvision::Blobs>(topic,nh)->blobs[0].x;
		
		if(x < (2*image_width/3) && x > (image_width/3))
		{
			ROS_INFO("Centered object");
			return 2;
		}
		else if(x > (2*image_width/3))
		{
			ROS_INFO("Right-positioned object");
			return 3;
		}
		else if(x < (image_width/3))
		{
			ROS_INFO("Left-positioned object");
			return 1;
		}
	}
	ROS_INFO("No object");

	return 0;
}

int detectPosition(ros::NodeHandle nh)
{
	std::string topic;
	topic= "blobs";

	if(ros::topic::waitForMessage<cmvision::Blobs>(topic,nh)->blob_count > 0)
	{
		int width= ros::topic::waitForMessage<cmvision::Blobs>(topic,nh)->blobs[0].right - ros::topic::waitForMessage<cmvision::Blobs>(topic,nh)->blobs[0].left;
		int height= ros::topic::waitForMessage<cmvision::Blobs>(topic,nh)->blobs[0].bottom - ros::topic::waitForMessage<cmvision::Blobs>(topic,nh)->blobs[0].top;
		

		if(width>height){
			ROS_INFO("horizontal");
			return 0;
		}else{
			ROS_INFO("vertical");
			return 1;
		}
	}
	
	else return -1;
}

int detect_object_on_box(ros::NodeHandle nh, ros::Publisher pub){
	
	turnHead(pub,0);
	sleep(1);	

	int object= detectObject(nh);
	int position;

	if(object == 2){
		sleep(1);
		position= detectPosition(nh);
		if(position == 0){
			return 10;
		}else if(position == 1){
			return 11;
		}
	}
	else
	{
		turnHead(pub,30);
		sleep(1);
		object=detectObject(nh);
		if(object==2)
		{
			sleep(1);
			position= detectPosition(nh);
			if(position == 0){
				return 20;
			}else if(position == 1){
				return 21;
			}
		}
	}

	turnHead(pub,-30);
	sleep(1);
	object=detectObject(nh);
	if(object == 2)
	{
		sleep(1);
		position= detectPosition(nh);
		if(position == 0)
		{
			return 30;
		}
		else if(position == 1)
		{
			return 31;
		}
	}
	
	turnHead(pub,0);
	sleep(1);	
	return 0;
	
}

		

/**
 * OBJECTATRANGE
 */
int objectAtRange(ros::NodeHandle nh,unsigned int size)
{

	std::string topic;
	topic= "blobs";

	if(ros::topic::waitForMessage<cmvision::Blobs>(topic,nh)->blobs[0].area > size)
	{
		return 1;
	}
		
	usleep(1000000);

	return 0;
}

/**
 * FOLLOWOBJECT
 */
int followObject (ros::NodeHandle nh, ros::Publisher pub)
{
	int aux= detectObject(nh);
	while(detectObject(nh) != 0)
	{
	if(aux == 2)
	{
		while(aux == 2){
			
			ROS_INFO("Walking towards object");
			walkForward(pub);
			aux=detectObject(nh);
		}
		sleep(1);
	}

	if(aux == 1)
	{
		while(aux == 1)
		{	
			usleep(500000);
			ROS_INFO("Turning left");
			turnLeft(pub);
			aux= detectObject(nh);
		}
		stopWalking(pub);
		sleep(1);
	}
	
	if(aux == 3)
	{
		while(aux == 3)
		{
			usleep(500000);
			ROS_INFO("Turning right");
			turnRight(pub);
			aux= detectObject(nh);
		}
		stopWalking(pub);
		sleep(1);
	}
	
	sleep(1);
	}
	
	stopWalking(pub);
	return 0;
}

/**
 * PASSOBSTACLE
 */
int passObstacle(ros::NodeHandle nh,ros::Publisher pub)
{
	int sonar= getSonarInfo(nh);	
	if(sonar>0){
		walkSidewaysDistance(pub,1,1);
		sleep(2);
		walkDistance(pub,1,0);
		return 1;
	}
	return 0;
	
}

/**
 * MOVE_BODY
 */
void move_body(ros::Publisher pub, std::string names[], float joints[]){
	
	nao_msgs::JointAnglesWithSpeed msg_body;
	
	std::vector<std::string> jnames(names, names+sizeof(names));
	std::vector<float> jangles(joints, joints+sizeof(joints));	

	msg_body.joint_names= jnames;
	msg_body.joint_angles= jangles;
	msg_body.speed= 0.2;

	for(int i=0; i< 26; i++){

	ROS_INFO(" - %f", jangles[i]);
	}
	pub.publish(msg_body);
	sleep(5);
}

void initPose(ros::Publisher pub){
	
	nao_msgs::JointAnglesWithSpeed msg_body;

	const float arr[] = {0.0,0.0, 1.39, 0.34, -1.39, -1.04, 0.0, 0.0, 0.0, 0.0, -0.43, 0.69, -0.34, 0.0, 0.0, 0.0, -0.43, 0.69, -0.34, 0.0, 1.39, -0.34, 1.39, 1.04, 0.0, 0.0};
	const int size = sizeof( arr ) / sizeof ( *arr );
	
	std::vector<float> jangles(arr, arr+size);
	
	std::vector<std::string> jnames(1,"Body");

	msg_body.joint_names= jnames;
	msg_body.joint_angles=jangles;
	msg_body.speed= 0.05;

	pub.publish(msg_body);
	sleep(5);
}

void zeroPose(ros::Publisher pub){
	
	nao_msgs::JointAnglesWithSpeed msg_body;	

	std::vector<float> jangles(26, 0.0);
	std::vector<std::string> jnames(1,"Body");

	msg_body.joint_names= jnames;
	msg_body.joint_angles=jangles;
	msg_body.speed= 0.05;

	pub.publish(msg_body);
	sleep(5);
}

void standPose(ros::Publisher pub){
	

}

void arm_to_table_grasppos_fromside(ros::Publisher pub, int arm){
	if(arm==1){
	openHand(pub, 1);
	usleep(500000);
	rollArm(pub, 1, 90);
	usleep(500000);
	liftArm(pub,1,5);
	usleep(500000);
	stretchArm(pub,1,-20);
	usleep(500000);
	rollWrist(pub,1,-102);
	usleep(500000);
	rollElbow(pub,1,48);
	usleep(500000);
	rollArm(pub,1,20);
	usleep(1500000);
	closeHand(pub, 1);
	usleep(500000);
	}
	else if(arm==0){
	openHand(pub, 0);
	usleep(1000000);
	rollArm(pub, 0, -90);
	usleep(1000000);
	liftArm(pub,0,5);
	usleep(1000000);
	stretchArm(pub,0,20);
	usleep(1000000);
	rollWrist(pub,0,102);
	usleep(1000000);
	rollElbow(pub,0,-48);
	usleep(1000000);
	rollArm(pub,0,-20);
	usleep(1500000);
	closeHand(pub,0);
	usleep(500000);
	}

	
}


void arm_to_standingpos_fromside(ros::Publisher pub, int arm){
	if(arm==1){
	stretchArm(pub,1,-0.5);
	usleep(500000);
	rollElbow(pub,1,-69.9);
	usleep(500000);
	rollWrist(pub,1,5.4);
	usleep(500000);
	rollArm(pub, 1, 90);
	usleep(500000);
	liftArm(pub, 1, 80);	
	usleep(500000);
	moveFullArm(pub, 1, 7, 85, -25);
	usleep(1500000);
	}else if(arm==0){
	stretchArm(pub,0,0.5);
	usleep(500000);
	rollElbow(pub,0,69.9);
	usleep(500000);
	rollWrist(pub,0,5.4);
	usleep(500000);
	rollArm(pub, 0, -90);
	usleep(500000);
	liftArm(pub, 0, 80);	
	usleep(500000);
	moveFullArm(pub, 0, -7, 85, 25);
	usleep(1500000);
	}

}

void arm_to_table_grasppos_fromtop(ros::Publisher pub, int arm,int soltar){
	
	if(arm==1){
	if(!soltar){
	openHand(pub, 1);
	usleep(1500000);
	}
	stretchArm(pub, 1, -0.5);
	usleep(1500000);
	rollArm(pub, 1, 90);
	usleep(1500000);
	liftArm(pub, 1, -90);
	usleep(1500000);
	rollArm(pub, 1, 5);
	usleep(1500000);
	rollElbow(pub, 1, 0.0);
	usleep(1500000);
	
	if(soltar){
	liftArm(pub, 1, 5);
	usleep(1500000);
	openHand(pub,1);
	}else{
	liftArm(pub, 1, 24);
	usleep(1500000);
	closeHand(pub,1);
	}
	usleep(1500000);
	}

	else if(arm==0){
	if(!soltar){
	openHand(pub, 0);
	usleep(2000000);
	}
	stretchArm(pub, 0, 0.5);
	usleep(2000000);
	rollArm(pub, 0, -90);
	usleep(2000000);
	liftArm(pub, 0, -90);
	usleep(2000000);
	rollArm(pub, 0, -5);
	usleep(2000000);
	rollElbow(pub, 0, 0.0);
	usleep(2000000);
	
	if(soltar){
	liftArm(pub, 0, 5);
	usleep(2000000);
	openHand(pub,0);
	
	}else{
	liftArm(pub, 0, 24);
	usleep(2000000);
	closeHand(pub,0);
	}
	usleep(1500000);	
	}
}

void arm_to_standingpos_fromtop(ros::Publisher pub, int arm){
	if(arm==1){
	liftArm(pub, 1, -90);
	usleep(1500000);
	rollElbow(pub, 1, -67);
	usleep(500000);
	rollArm(pub, 1, 90);
	usleep(1500000);
	liftArm(pub, 1, 90);
	usleep(1500000);
	rollArm(pub,1, 8);
	usleep(1500000);
	stretchArm(pub,1, -25);
	usleep(1500000);
	}
	else if(arm==0){
	liftArm(pub, 0, -90);
	usleep(1500000);
	rollElbow(pub, 0, 67);
	usleep(500000);
	rollArm(pub, 0, -90);
	usleep(1500000);
	liftArm(pub, 0, 90);
	usleep(1500000);
	rollArm(pub,0, -8);
	usleep(1500000);
	stretchArm(pub,0, 25);
	usleep(1500000);
	}
}
	



void pickup_object_from_box(ros::NodeHandle nh, ros::Publisher pub)
{
	int pos= detect_object_on_box(nh,pub);
	if(pos == 20)
	{
		arm_to_table_grasppos_fromtop(pub,1,0);
		sleep(2);
		arm_to_standingpos_fromtop(pub,1);
	}
	if(pos == 21)
	{
		arm_to_table_grasppos_fromside(pub,1);
		sleep(2);
		arm_to_standingpos_fromside(pub,1);
	}
	if(pos == 30)
	{
		arm_to_table_grasppos_fromtop(pub,0,0);
		sleep(2);
		arm_to_standingpos_fromtop(pub,0);
	}
	if(pos == 31)
	{
		arm_to_table_grasppos_fromside(pub,0);
		sleep(2);
		arm_to_standingpos_fromside(pub,0);
	}
}

void walk_and_speak(ros::Publisher pubj, ros::Publisher pubs, ros::Publisher pubh)
{
	walkDistance(pubs,0.5,0);
	sleep(3);
	moveFullArm(pubj, 0, -63,28,90);
	sleep(2);
	stretchArm(pubj,0,45);
	sleep(1);
	stretchArm(pubj,0,90);
	sleep(1);
	stretchArm(pubj,0,45);
	sleep(1);
}
int main(int argc, char **argv)
{   	
	
	ros::init(argc, argv, "prueba"); 
	
	ros::NodeHandle nh;
	ros::Rate loop_rate(10);
	
	if ((nh.hasParam("IP")) && (nh.hasParam("port")))
	{
		nh.getParam("IP", server_ip);
		ROS_INFO("IP: %s", server_ip.c_str());
		nh.getParam("port", server_port);
		ROS_INFO("Port: %s", server_port.c_str());
		nh.getParam("mode", server_port);
		ROS_INFO("MODE: %s", server_port.c_str());

	}
	else
	{
		ROS_ERROR("Params in launch file are not correct");
		return 1;
	}

	
	ros::Publisher speech_pub = nh.advertise<std_msgs::String>("speech", 1);
	ros::Publisher cmdVel_pub = nh.advertise<geometry_msgs::Twist>("cmd_vel", 1);
	ros::Publisher jointAngles_pub = nh.advertise<nao_msgs::JointAnglesWithSpeed>("joint_angles", 1); 
	ros::Publisher bodyHand_pub = nh.advertise<body_msgs::Hand>("hand", 1);
	
	
	/*PARA NO PERDER EL PRIMER MENSAJE*/
	ROS_INFO("STARTING CONTROLLER...");
	recieveMessage(cmdVel_pub);
	sleep(1);
	ROS_INFO("CONTROLLER RUNNING...");	
	

	while (ros::ok())
	{
	/**
	 * EXPERIMENTO 1
	 */
	
	pickup_object_from_box(nh, jointAngles_pub);
	
	/**
	 * EXPERIMENTO 2
	 */
	
	/*walk_and_speak(jointAngles_pub,cmdVel_pub,speech_pub);*/
	

	/**
	 * EXPERIMENTO 3
	 */
	/*while(!getSonarInfo(nh)){
		walkDistance(cmdVel_pub,0.2,0);
	}
	passObstacle(nh, cmdVel_pub);*/
	

	/**
	 * EXPERIMENTO 4
	 */
	
	/*arm_to_table_grasppos_fromtop(jointAngles_pub,0,0);
	sleep(2);
	arm_to_standingpos_fromtop(jointAngles_pub,0);
	sleep(2);
	walkDistance(cmdVel_pub,0.05,1);
	sleep(2);
	walkDistance(cmdVel_pub,0.2,1);
	sleep(2);
	turnAngle(cmdVel_pub,-90);
	sleep(2);
	walkDistance(cmdVel_pub,0.6,0);
	sleep(2);
	say(speech_pub, "Box");
	sleep(2);
	arm_to_table_grasppos_fromtop(jointAngles_pub,0,1);
	sleep(2);
	arm_to_standingpos_fromtop(jointAngles_pub,0);*/
	

	return 0;
	}

	return 0;
}



