/*
 *
 * Copyright 2012 Moisés Martínez, Ignacio Elizaga. Universidad Carlos III de Madrid
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the University Carlos III de madrid nor the names 
 *			of its contributors may be used to endorse or promote products derived 
 *       from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#include "naoRobot_2.h"

#define RADIAN 0.0174532925

naoRobot::naoRobot(string value)
{
	ros::NodeHandle nh;

	name 		= value;
	pos_x 		= 0.0;	
	pos_y 		= 0.0;
	direction 	= 0; //0 grades
	batery		= 50.0;

	memset(sonar, 0, SONAR_VALUES);


	speed_pub 		= nh.advertise<geometry_msgs::Twist>("/cmd_vel", 1);
	speech_pub		= nh.advertise<std_msgs::String>("/speech", 1);
	joints_pub		= nh.advertise<nao_msgs::JointAnglesWithSpeed>("/joint_angles", 1);
};

naoRobot::~naoRobot()
{
};

/**
 * walk_forward
 */
void naoRobot::walk_forward()
{
	geometry_msgs::Twist speed;

	speed.linear.x    = 1;
	speed.linear.y    = 0;
	speed.linear.z    = 0;
	speed.angular.x   = 0;
	speed.angular.y   = 0;
	speed.angular.z   = 0;

	speed_pub.publish(speed);
}

/**
 * walk_backwards
 */
void naoRobot::walk_backwards()
{
	geometry_msgs::Twist speed;

	speed.linear.x    = -1;
	speed.linear.y    = 0;
	speed.linear.z    = 0;
	speed.angular.x   = 0;
	speed.angular.y   = 0;
	speed.angular.z   = 0;

	speed_pub.publish(speed);
}

/**
 * walk_sideways_right
 */
void naoRobot::walk_sideways_right()
{
	geometry_msgs::Twist speed;

	speed.linear.x    = 0;
	speed.linear.y    = -1;
	speed.linear.z    = 0;
	speed.angular.x   = 0;
	speed.angular.y   = 0;
	speed.angular.z   = 0;

	speed_pub.publish(speed);
}

/**
 * walk_sideways_left
 */
void naoRobot::walk_sideways_left()
{
	geometry_msgs::Twist speed;

	speed.linear.x    = 0;
	speed.linear.y    = 1;
	speed.linear.z    = 0;
	speed.angular.x   = 0;
	speed.angular.y   = 0;
	speed.angular.z   = 0;

	speed_pub.publish(speed);
}

/**
 * stop
 */
void naoRobot::stop()
{
	geometry_msgs::Twist speed;

	speed.linear.x    = 0;
	speed.linear.y    = 0;
	speed.linear.z    = 0;
	speed.angular.x   = 0;
	speed.angular.y   = 0;
	speed.angular.z   = 0;

	speed_pub.publish(speed);
}

/**
 * turn_right
 */
void naoRobot::turn_right()
{
	geometry_msgs::Twist speed;

	speed.linear.x    = 0;
	speed.linear.y    = 0;
	speed.linear.z    = 0;
	speed.angular.x   = 0;
	speed.angular.y   = 0;
	speed.angular.z   = -1;

	speed_pub.publish(speed);
}

/**
 * turn_left
 */
void naoRobot::turn_left()
{
	geometry_msgs::Twist speed;

	speed.linear.x    = 0;
	speed.linear.y    = 0;
	speed.linear.z    = 0;
	speed.angular.x   = 0;
	speed.angular.y   = 0;
	speed.angular.z   = 1;

	speed_pub.publish(speed);
}

/**
 * turn_head		MAX ANGLE = 119.5	MIN ANGLE = -119.5
 */
void naoRobot::turn_head(int angle)
{
	float angle_rad= angle*RADIAN;

	nao_msgs::JointAnglesWithSpeed joint_msg;

	std::vector<std::string> jnames(1,"HeadYaw");
	std::vector<float> jangles(1,angle_rad);

	joint_msg.joint_names= jnames;
	joint_msg.joint_angles=jangles;
	joint_msg.speed= 0.25;

	joints_pub.publish(joint_msg);
}

/**
 * bow_head		MAX ANGLE = 29.5	MIN ANGLE = -38.5
 */
void naoRobot::bow_head(int angle)
{
	float angle_rad= angle*RADIAN;

	nao_msgs::JointAnglesWithSpeed joint_msg;

	std::vector<std::string> jnames(1,"HeadPitch");
	std::vector<float> jangles(1,angle_rad);

	joint_msg.joint_names= jnames;
	joint_msg.joint_angles=jangles;
	joint_msg.speed= 0.25;

	joints_pub.publish(joint_msg);
}

/**
 * open_hand
 */
void naoRobot::open_hand(int hand)
{

	size_t size= 1;

	nao_msgs::JointAnglesWithSpeed joint_msg;

	std::vector<std::string> jnames(size);
	std::vector<float> jangles(1,1);

	if(hand==1)
	{
		jnames[0]="LHand";
	}
	else if(hand==0)
	{
		jnames[0]="RHand";
	}
	
	joint_msg.joint_names= jnames;
	joint_msg.joint_angles=jangles;
	joint_msg.speed= 1;

	joints_pub.publish(joint_msg);
}


/**
 * close_hand
 */
void naoRobot::close_hand(int hand)
{

	size_t size= 1;

	nao_msgs::JointAnglesWithSpeed joint_msg;

	std::vector<std::string> jnames(size);
	std::vector<float> jangles(1,0);

	if(hand==1)
	{
		jnames[0]="LHand";
	}
	else if(hand==0)
	{
		jnames[0]="RHand";
	}
	
	joint_msg.joint_names= jnames;
	joint_msg.joint_angles=jangles;
	joint_msg.speed= 1;

	joints_pub.publish(joint_msg);
}

/**
 * say
 */
void naoRobot::say(std::string speech)
{
	std_msgs::String msg_speech;

	msg_speech.data=speech;

	speech_pub.publish(msg_speech);
}

/**
 * stretch_arm			
 * LEFT [-88.5, -2]
 * RIGHT [2,88.5]
 */
void naoRobot::stretch_arm(int arm, double angle)
{
	size_t size= 1;

	nao_msgs::JointAnglesWithSpeed msg_arm;

	std::vector<std::string> jnames(size);
	std::vector<float> jangles(size);

	if(arm==1)
	{
		jnames[0]="LElbowRoll";
		
		jangles[0]=angle*RADIAN;
	}
	else if(arm==0)
	{
		jnames[0]="RElbowRoll";
		
		jangles[0]=angle*RADIAN;
	}
	
	msg_arm.joint_names= jnames;
	msg_arm.joint_angles=jangles;
	msg_arm.speed= 0.25;

	joints_pub.publish(msg_arm);
}

/**
 * lift_arm			
 * LEFT [-119.5, 119.5]
 * RIGHT [-119.5, 119.5]
 */
void naoRobot::lift_arm(int arm, double angle)
{
	size_t size= 1;

	nao_msgs::JointAnglesWithSpeed msg_arm;

	std::vector<std::string> jnames(size);
	std::vector<float> jangles(size);

	if(arm==1)
	{
		jnames[0]="LShoulderPitch";
		
		jangles[0]=angle*RADIAN;
	}
	else if(arm==0)
	{
		jnames[0]="RShoulderPitch";
		
		jangles[0]=angle*RADIAN;
	}
	
	msg_arm.joint_names= jnames;
	msg_arm.joint_angles=jangles;
	msg_arm.speed= 0.25;

	joints_pub.publish(msg_arm);
}

/**
 * roll_arm			
 * LEFT [-18, 76]
 * RIGHT [-76, 18]
 */
void naoRobot::roll_arm(int arm, double angle)
{
	size_t size= 1;

	nao_msgs::JointAnglesWithSpeed msg_arm;

	std::vector<std::string> jnames(size);
	std::vector<float> jangles(size);

	if(arm==1)
	{
		jnames[0]="LShoulderRoll";
		
		jangles[0]=angle*RADIAN;
	}
	else if(arm==0)
	{
		jnames[0]="RShoulderRoll";
		
		jangles[0]=angle*RADIAN;
	}
	
	msg_arm.joint_names= jnames;
	msg_arm.joint_angles=jangles;
	msg_arm.speed= 0.25;

	joints_pub.publish(msg_arm);
}

/**
 * roll_wrist		
 * LEFT [-104.5, 104.5]
 * RIGHT [-104.5, 104.5]
 */
void naoRobot::roll_wrist(int arm, double angle)
{
	size_t size= 1;

	nao_msgs::JointAnglesWithSpeed msg_arm;

	std::vector<std::string> jnames(size);
	std::vector<float> jangles(size);

	if(arm==1)
	{
		jnames[0]="LWristYaw";
		
		jangles[0]=angle*RADIAN;
	}
	else if(arm==0)
	{
		jnames[0]="RWristYaw";
		
		jangles[0]=angle*RADIAN;
	}
	
	msg_arm.joint_names= jnames;
	msg_arm.joint_angles=jangles;
	msg_arm.speed= 0.25;

	joints_pub.publish(msg_arm);
}

/**
 * roll_elbow
 * LEFT [-104.5, 104.5]
 * RIGHT [-104.5, 104.5]
 */
void naoRobot::roll_elbow(int arm, double angle)
{
	size_t size= 1;

	nao_msgs::JointAnglesWithSpeed msg_arm;

	std::vector<std::string> jnames(size);
	std::vector<float> jangles(size);

	if(arm==1)
	{
		jnames[0]="LElbowYaw";
		
		jangles[0]=angle*RADIAN;
	}
	else if(arm==0)
	{
		jnames[0]="RElbowYaw";
		
		jangles[0]=angle*RADIAN;
	}
	
	msg_arm.joint_names= jnames;
	msg_arm.joint_angles=jangles;
	msg_arm.speed= 0.25;

	joints_pub.publish(msg_arm);
}

/**
 * start_moving_bug
 */
void naoRobot::start_moving_bug()
{	
	
	walk_forward();

	usleep(1000000);

	stop();

}

/**
 * move_full_arm
 */
void naoRobot::move_full_arm(int arm, double shoulderRollAngle, double shoulderPitchAngle, double elbowRollAngle)
{
	size_t size= 3;

	nao_msgs::JointAnglesWithSpeed msg_arm;

	std::vector<std::string> jnames(size);
	std::vector<float> jangles(size);

	if(arm==1)
	{
		jnames[0]="LShoulderRoll";
		jnames[1]="LShoulderPitch";
		jnames[2]="LElbowRoll";

		jangles[0]=shoulderRollAngle*(0.0174532925);
		jangles[1]=shoulderPitchAngle*(0.0174532925);
		jangles[2]=elbowRollAngle*(0.0174532925);
	}
	else if(arm==0)
	{
		jnames[0]="RShoulderRoll";
		jnames[1]="RShoulderPitch";
		jnames[2]="RElbowRoll";

		jangles[0]=shoulderRollAngle*-(0.0174532925);
		jangles[1]=shoulderPitchAngle*(0.0174532925);
		jangles[2]=elbowRollAngle*(0.0174532925);
	}
	
	msg_arm.joint_names= jnames;
	msg_arm.joint_angles=jangles;
	msg_arm.speed= 0.25;

	joints_pub.publish(msg_arm);
}

/**
 * walk_distance
 */
void naoRobot::walk_distance(double distance,int direction)
{	
	
	if(direction==0)
	{
		walk_forward();

		usleep(10*distance*1000000);

		stop();

		usleep(1000000);
	}
	else if(direction==1)
	{
		walk_backwards();

		usleep(10*distance*1000000);
		
		stop();

		usleep(1000000);
	}
}

/**
 * walk_sideways_distance
 */
void naoRobot::walk_sideways_distance(double distance,int direction)
{	
	
	if(direction==0)
	{
		walk_sideways_right();

		usleep(10*distance*1000000);

		stop();

		usleep(1000000);
	}
	else if(direction==1)
	{
		walk_sideways_left();

		usleep(10*distance*1000000);
		
		stop();

		usleep(1000000);
	}
}

/**
 * turn_angle
 */
void naoRobot::turn_angle(double angle)
{	
	if(angle>0)
	{
		turn_right(); 

		usleep(2*angle*0.0305*1000000);

		stop();

		usleep(1500000);
	}
	else
	{
		turn_left();

		usleep(2*angle*-0.0305*1000000);

		stop();

		usleep(1500000);
	}
}

/**
 * get_sonar_info
 */
int naoRobot::get_sonar_info()
{

	std::string topic;
	topic= "sonar_odometry";
	bool left=false;
	bool right=false;

	float valL1=ros::topic::waitForMessage<nao_msgs::SonarOdometry>(topic,nh)->left1;
	float valL2=ros::topic::waitForMessage<nao_msgs::SonarOdometry>(topic,nh)->left1;
	float valL3=ros::topic::waitForMessage<nao_msgs::SonarOdometry>(topic,nh)->left1;
	float valL4=ros::topic::waitForMessage<nao_msgs::SonarOdometry>(topic,nh)->left1;
	float valL5=ros::topic::waitForMessage<nao_msgs::SonarOdometry>(topic,nh)->left1;
	
	float valL= (valL1+valL2+valL3+valL4+valL5)/5;

	float valR1=ros::topic::waitForMessage<nao_msgs::SonarOdometry>(topic,nh)->right1;
	float valR2=ros::topic::waitForMessage<nao_msgs::SonarOdometry>(topic,nh)->right1;
	float valR3=ros::topic::waitForMessage<nao_msgs::SonarOdometry>(topic,nh)->right1;
	float valR4=ros::topic::waitForMessage<nao_msgs::SonarOdometry>(topic,nh)->right1;
	float valR5=ros::topic::waitForMessage<nao_msgs::SonarOdometry>(topic,nh)->right1;

	float valR= (valR1+valR2+valR3+valR4+valR5)/5;

	if(valL <1)
	{
		left= true;
	}
	if(valR < 1)
	{
		right= true;
	}

	if(!left && !right)
	{
		return 0;
	}
	else if(left && !right)
	{
		return 1;
	}
	else if(!left && right)
	{
		return 2;
	}
	else if(left && right)
	{
		return 3;
	}
	
	usleep(1500000);
	
	return -1;
}

/**
 * detect_object
 */
int naoRobot::detect_object()
{

	std::string topic;
	topic= "blobs";

	if(ros::topic::waitForMessage<cmvision::Blobs>(topic,nh)->blob_count > 0)
	{
		int image_width = ros::topic::waitForMessage<cmvision::Blobs>(topic,nh)->image_width;
		int x= ros::topic::waitForMessage<cmvision::Blobs>(topic,nh)->blobs[0].x;
		
		if(x < (2*image_width/3) && x > (image_width/3))
		{
			ROS_INFO("Centered object");
			return 2;
		}
		else if(x > (2*image_width/3))
		{
			ROS_INFO("Right-positioned object");
			return 3;
		}
		else if(x < (image_width/3))
		{
			ROS_INFO("Left-positioned object");
			return 1;
		}
	}
	ROS_INFO("No object");

	return 0;
}

/**
 * detect_position
 */
int naoRobot::detect_position()
{
	std::string topic;
	topic= "blobs";

	if(ros::topic::waitForMessage<cmvision::Blobs>(topic,nh)->blob_count > 0)
	{
		int width= ros::topic::waitForMessage<cmvision::Blobs>(topic,nh)->blobs[0].right - ros::topic::waitForMessage<cmvision::Blobs>(topic,nh)->blobs[0].left;
		int height= ros::topic::waitForMessage<cmvision::Blobs>(topic,nh)->blobs[0].bottom - ros::topic::waitForMessage<cmvision::Blobs>(topic,nh)->blobs[0].top;
		

		if(width>height)
		{
			ROS_INFO("horizontal");
			return 0;
		}
		else
		{
			ROS_INFO("vertical");
			return 1;
		}
	}
	
	else return -1;
}

/**
 * detect_object_on_box
 */
int naoRobot::detect_object_on_box(){
	
	turn_head(0);
	sleep(1);	

	int object= detect_object();
	int position;

	if(object == 2){
		sleep(1);
		position= detect_position();

		if(position == 0)
		{
			return 10;
		}
		else if(position == 1)
		{
			return 11;
		}
	}
	else
	{
		turn_head(30);
		sleep(1);
		object= detect_object();

		if(object==2)
		{
			sleep(1);
			position= detect_position();
			if(position == 0)
			{
				return 20;
			}
			else if(position == 1)
			{
				return 21;
			}
		}
	}

	turn_head(-30);
	sleep(1);
	object= detect_object();

	if(object == 2)
	{
		sleep(1);
		position= detect_position();

		if(position == 0)
		{
			return 30;
		}
		else if(position == 1)
		{
			return 31;
		}
	}
	
	turn_head(0);
	sleep(1);	
	return 0;
	
}

/**
 * object_at_range
 */
int naoRobot::object_at_range(unsigned int size)
{

	std::string topic;
	topic= "blobs";

	if(ros::topic::waitForMessage<cmvision::Blobs>(topic,nh)->blobs[0].area > size)
	{
		return 1;
	}
		
	usleep(1000000);

	return 0;
}

/**
 * FOLLOWOBJECT
 */
int naoRobot::follow_object ()
{
	int aux=  detect_object();
	while( detect_object() != 0)
	{
		if(aux == 2)
		{
			while(aux == 2){

				ROS_INFO("Walking towards object");
				walk_forward();
				aux= detect_object();
			}
			sleep(1);
		}

		if(aux == 1)
		{
			while(aux == 1)
			{	
				usleep(500000);
				ROS_INFO("Turning left");
				turn_left();
				aux= detect_object();
			}
			stop();
			sleep(1);
		}

		if(aux == 3)
		{
			while(aux == 3)
			{
				usleep(500000);
				ROS_INFO("Turning right");
				turn_right();
				aux= detect_object();
			}
			stop();
			sleep(1);
		}

		sleep(1);
	}

	stop();
	return 0;
}

/**
 * pass_obstacle
 */
int naoRobot::pass_obstacle()
{
	int sonar= get_sonar_info();	
	if(sonar>0)
	{
		while(sonar>0)
		{	
			
			walk_sideways_right();
			sleep(1);
			sonar= get_sonar_info();
		}
		walk_distance(0.4,0);
		return 1;
	}

	ROS_INFO("NO OBSTACLE FOUND");
	return 0;
	
}

/**
 * move_body
 * CUIDADO CON ESTA PRIMITIVA EL ROBOT PUEDE DESEQUILIBRARSE
 */
void naoRobot::move_body(std::string names[], float joints[]){
	
	nao_msgs::JointAnglesWithSpeed msg_body;
	
	std::vector<std::string> jnames(names, names+sizeof(names));
	std::vector<float> jangles(joints, joints+sizeof(joints));	

	msg_body.joint_names= jnames;
	msg_body.joint_angles= jangles;
	msg_body.speed= 0.2;

	for(int i=0; i< 26; i++){

	ROS_INFO(" - %f", jangles[i]);
	}
	joints_pub.publish(msg_body);
	sleep(5);
}

/**
 * init_pose
 * CUIDADO CON ESTA PRIMITIVA EL ROBOT PUEDE DESEQUILIBRARSE
 */
void naoRobot::init_pose(){
	
	nao_msgs::JointAnglesWithSpeed msg_body;

	const float arr[] = {0.0,0.0, 1.39, 0.34, -1.39, -1.04, 0.0, 0.0, 0.0, 0.0, -0.43, 0.69, -0.34, 0.0, 0.0, 0.0, -0.43, 0.69, -0.34, 0.0, 1.39, -0.34, 1.39, 1.04, 0.0, 0.0};
	const int size = sizeof( arr ) / sizeof ( *arr );
	
	std::vector<float> jangles(arr, arr+size);
	
	std::vector<std::string> jnames(1,"Body");

	msg_body.joint_names= jnames;
	msg_body.joint_angles=jangles;
	msg_body.speed= 0.05;

	joints_pub.publish(msg_body);
	sleep(5);
}

/**
 * zero_pose
 * CUIDADO CON ESTA PRIMITIVA EL ROBOT PUEDE DESEQUILIBRARSE
 */
void naoRobot::zeroPose(){
	
	nao_msgs::JointAnglesWithSpeed msg_body;	

	std::vector<float> jangles(26, 0.0);
	std::vector<std::string> jnames(1,"Body");

	msg_body.joint_names= jnames;
	msg_body.joint_angles=jangles;
	msg_body.speed= 0.05;

	joints_pub.publish(msg_body);
	sleep(5);
}

/**
 * arm_to_table_grasppos_fromside
 */
void naoRobot::arm_to_table_grasppos_fromside(int arm){
	if(arm==1){
		open_hand(1);
		usleep(500000);
		roll_arm(1, 90);
		usleep(500000);
		lift_arm(1,5);
		usleep(500000);
		stretch_arm(1,-20);
		usleep(500000);
		roll_wrist(1,-102);
		usleep(500000);
		roll_elbow(1,48);
		usleep(500000);
		roll_arm(1,20);
		usleep(1500000);
		close_hand(1);
		usleep(500000);
	}
	else if(arm==0){
		open_hand(0);
		usleep(1000000);
		roll_arm(0, -90);
		usleep(1000000);
		lift_arm(0,5);
		usleep(1000000);
		stretch_arm(0,20);
		usleep(1000000);
		roll_wrist(0,102);
		usleep(1000000);
		roll_elbow(0,-48);
		usleep(1000000);
		roll_arm(0,-20);
		usleep(1500000);
		close_hand(0);
		usleep(500000);
	}	
}

/**
 * arm_to_standingpos_fromside
 */
void naoRobot::arm_to_standingpos_fromside(int arm){
	if(arm==1){
		stretch_arm(1,-0.5);
		usleep(500000);
		roll_elbow(1,-69.9);
		usleep(500000);
		roll_wrist(1,5.4);
		usleep(500000);
		roll_arm(1, 90);
		usleep(500000);
		lift_arm(1, 80);	
		usleep(500000);
		move_full_arm(1, 7, 85, -25);
		usleep(1500000);
	}else if(arm==0){
		stretch_arm(0,0.5);
		usleep(500000);
		roll_elbow(0,69.9);
		usleep(500000);
		roll_wrist(0,5.4);
		usleep(500000);
		roll_arm(0, -90);
		usleep(500000);
		lift_arm(0, 80);	
		usleep(500000);
		move_full_arm(0, -7, 85, 25);
		usleep(1500000);
	}

}

/**
 * arm_to_table_grasppos_fromtop
 */
void naoRobot::arm_to_table_grasppos_fromtop(int arm, int soltar){

	if(arm==1){
		if(!soltar){
			open_hand(1);
			usleep(1500000);
		}
		stretch_arm(1, -0.5);
		usleep(1500000);
		roll_arm(1, 90);
		usleep(1500000);
		lift_arm(1, -90);
		usleep(1500000);
		roll_arm(1, 5);
		usleep(1500000);
		roll_elbow(1, 0.0);
		usleep(1500000);

		if(soltar){
			lift_arm(1, 5);
			usleep(1500000);
			open_hand(1);
		}else{
			lift_arm(1, 24);
			usleep(1500000);
			close_hand(1);
		}
		usleep(1500000);
	}

	else if(arm==0){
		if(!soltar){
			open_hand(0);
			usleep(2000000);
		}
		stretch_arm(0, 0.5);
		usleep(2000000);
		roll_arm(0, -90);
		usleep(2000000);
		lift_arm(0, -90);
		usleep(2000000);
		roll_arm(0, -5);
		usleep(2000000);
		roll_elbow(0, 0.0);
		usleep(2000000);

		if(soltar){
			lift_arm(0, 5);
			usleep(2000000);
			open_hand(0);

		}else{
			lift_arm(0, 24);
			usleep(2000000);
			close_hand(0);
		}
		usleep(1500000);	
	}
}

/**
 * arm_to_standingpos_fromtop
 */
void naoRobot::arm_to_standingpos_fromtop(int arm){
	if(arm==1){
		lift_arm(1, -90);
		usleep(1500000);
		roll_elbow(1, -67);
		usleep(500000);
		roll_arm(1, 90);
		usleep(1500000);
		lift_arm(1, 90);
		usleep(1500000);
		roll_arm(1, 8);
		usleep(1500000);
		stretch_arm(1, -25);
		usleep(1500000);
	}
	else if(arm==0){
		lift_arm(0, -90);
		usleep(1500000);
		roll_elbow(0, 67);
		usleep(500000);
		roll_arm(0, -90);
		usleep(1500000);
		lift_arm(0, 90);
		usleep(1500000);
		roll_arm(0, -8);
		usleep(1500000);
		stretch_arm(0, 25);
		usleep(1500000);
	}
}

/**
 * pickup_object_from_box
 */
void naoRobot::pickup_object_from_box()
{
	int pos= naoRobot::detect_object_on_box();
	if(pos == 20)
	{
		naoRobot::arm_to_table_grasppos_fromtop(1,0);
		sleep(2);
		naoRobot::arm_to_standingpos_fromtop(1);
	}
	if(pos == 21)
	{
		naoRobot::arm_to_table_grasppos_fromside(1);
		sleep(2);
		naoRobot::arm_to_standingpos_fromside(1);
	}
	if(pos == 30)
	{
		naoRobot::arm_to_table_grasppos_fromtop(0,0);
		sleep(2);
		naoRobot::arm_to_standingpos_fromtop(0);
	}
	if(pos == 31)
	{
		naoRobot::arm_to_table_grasppos_fromside(0);
		sleep(2);
		naoRobot::arm_to_standingpos_fromside(0);
	}
}
