/*
 *
 * Copyright 2012 Moisés Martínez, Ignacio Elizaga. Universidad Carlos III de Madrid
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the University Carlos III de madrid nor the names 
 *			of its contributors may be used to endorse or promote products derived 
 *       from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef NAOSTATE_H
#define NAOSTATE_H

#include <stdio.h>
#include <unistd.h>
#include <math.h>
#include <stdlib.h>

#include <vector>
#include <iostream>
#include <string>
#include <sstream>

#include <geometry_msgs/Twist.h>
#include <sensor_msgs/JointState.h>

#include <nao_msgs/TorsoOdometry.h>
#include <nao_msgs/TorsoIMU.h>
#include <nao_msgs/Bumper.h>
#include <nao_msgs/TactileTouch.h>

#include <geometry_msgs/Twist.h>
#include <nao_msgs/JointAnglesWithSpeed.h>
#include <nao_msgs/SonarOdometry.h>
#include <cmvision/Blobs.h>

//#include <tinyxml/tinyxml.h>

#include "ros/ros.h"
#include "std_msgs/String.h"

//#include "action.h"

#define ODOMETRY_VALUES 6
#define LEGS_VALUES 6
#define HEAD_VALUES 3
#define TORSO_VALUES 6
#define TORSOIMU_VALUES 7
#define BUMPERS_VALUES 6
#define TACTILE_VALUES 7
#define SONAR_VALUES 4

using namespace std;

static const string sonar_names [SONAR_VALUES] = { "LEFT1", "LEFT2", "RIGHT1", "RIGHT2"};

class naoRobot
{
	private:

		ros::NodeHandle nh;

		string name;

		float pos_x;
		float pos_y;
		int direction;
		float batery;

		string server_port;
		string server_ip;

		float sonar [SONAR_VALUES];

		ros::Subscriber sonar_sub;

		ros::Publisher speed_pub;
		ros::Publisher speech_pub;
		ros::Publisher joints_pub;


	public:

		naoRobot(string value);
		~naoRobot();

		void walk_forward();
		void walk_backwards();
		void walk_sideways_right();
		void walk_sideways_left();
		void stop();
		void turn_right();
		void turn_left();
		void turn_head(int angle);
		void bow_head(int angle);
		void open_hand(int hand);
		void close_hand(int hand);
		void say(std::string speech);
		void stretch_arm(int arm, double angle);
		void lift_arm(int arm, double angle);
		void roll_arm(int arm, double angle);
		void roll_wrist(int arm, double angle);
		void roll_elbow(int arm, double angle);
		void start_moving_bug();
		void move_full_arm(int arm, double shoulderRollAngle, double shoulderPitchAngle, double elbowRollAngle);
		void walk_distance(double distance,int direction);
		void walk_sideways_distance(double distance,int direction);
		void turn_angle(double angle);
		int get_sonar_info();
		int detect_object();
		int detect_position();
		int detect_object_on_box();
		int object_at_range(unsigned int size);
		int follow_object();
		int pass_obstacle();
		void move_body(std::string names[], float joints[]);
		void init_pose();
		void zeroPose();
		void arm_to_table_grasppos_fromside(int arm);
		void arm_to_standingpos_fromside(int arm);
		void arm_to_table_grasppos_fromtop(int arm, int soltar);
		void arm_to_standingpos_fromtop(int arm);
		void pickup_object_from_box();
};

#endif
